/*
 * Copyright (C) 2014 Alberts Muktupāvels
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef STICKYNOTES_GSETTINGS_H
#define STICKYNOTES_GSETTINGS_H

#define STICKYNOTES_SCHEMA "org.gnome.gnome-applets.stickynotes"

#define KEY_DEFAULT_WIDTH      "default-width"
#define KEY_DEFAULT_HEIGHT     "default-height"
#define KEY_DEFAULT_COLOR      "default-color"
#define KEY_DEFAULT_FONT_COLOR "default-font-color"
#define KEY_DEFAULT_FONT       "default-font"

#define KEY_STICKY           "sticky"
#define KEY_LOCKED           "locked"
#define KEY_DATE_FORMAT      "date-format"
#define KEY_USE_SYSTEM_COLOR "use-system-color"
#define KEY_USE_SYSTEM_FONT  "use-system-font"
#define KEY_FORCE_DEFAULT    "force-default"
#define KEY_DESKTOP_HIDE     "desktop-hide"
#define KEY_CONFIRM_DELETION "confirm-deletion"

#endif
