/*
 * Generated by gdbus-codegen 2.51.1. DO NOT EDIT.
 *
 * The license of this code is the same as for the source it was derived from.
 */

#ifndef __DBUS_BRIGHTNESS_H__
#define __DBUS_BRIGHTNESS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SettingsDaemon.Power.Screen */

#define DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN (dbus_settings_daemon_power_screen_get_type ())
#define DBUS_SETTINGS_DAEMON_POWER_SCREEN(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN, DBusSettingsDaemonPowerScreen))
#define DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN))
#define DBUS_SETTINGS_DAEMON_POWER_SCREEN_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN, DBusSettingsDaemonPowerScreenIface))

struct _DBusSettingsDaemonPowerScreen;
typedef struct _DBusSettingsDaemonPowerScreen DBusSettingsDaemonPowerScreen;
typedef struct _DBusSettingsDaemonPowerScreenIface DBusSettingsDaemonPowerScreenIface;

struct _DBusSettingsDaemonPowerScreenIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_step_down) (
    DBusSettingsDaemonPowerScreen *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_step_up) (
    DBusSettingsDaemonPowerScreen *object,
    GDBusMethodInvocation *invocation);

  gint  (*get_brightness) (DBusSettingsDaemonPowerScreen *object);

};

GType dbus_settings_daemon_power_screen_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *dbus_settings_daemon_power_screen_interface_info (void);
guint dbus_settings_daemon_power_screen_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void dbus_settings_daemon_power_screen_complete_step_up (
    DBusSettingsDaemonPowerScreen *object,
    GDBusMethodInvocation *invocation,
    gint new_percentage,
    gint output_id);

void dbus_settings_daemon_power_screen_complete_step_down (
    DBusSettingsDaemonPowerScreen *object,
    GDBusMethodInvocation *invocation,
    gint new_percentage,
    gint output_id);



/* D-Bus method calls: */
void dbus_settings_daemon_power_screen_call_step_up (
    DBusSettingsDaemonPowerScreen *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean dbus_settings_daemon_power_screen_call_step_up_finish (
    DBusSettingsDaemonPowerScreen *proxy,
    gint *out_new_percentage,
    gint *out_output_id,
    GAsyncResult *res,
    GError **error);

gboolean dbus_settings_daemon_power_screen_call_step_up_sync (
    DBusSettingsDaemonPowerScreen *proxy,
    gint *out_new_percentage,
    gint *out_output_id,
    GCancellable *cancellable,
    GError **error);

void dbus_settings_daemon_power_screen_call_step_down (
    DBusSettingsDaemonPowerScreen *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean dbus_settings_daemon_power_screen_call_step_down_finish (
    DBusSettingsDaemonPowerScreen *proxy,
    gint *out_new_percentage,
    gint *out_output_id,
    GAsyncResult *res,
    GError **error);

gboolean dbus_settings_daemon_power_screen_call_step_down_sync (
    DBusSettingsDaemonPowerScreen *proxy,
    gint *out_new_percentage,
    gint *out_output_id,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gint dbus_settings_daemon_power_screen_get_brightness (DBusSettingsDaemonPowerScreen *object);
void dbus_settings_daemon_power_screen_set_brightness (DBusSettingsDaemonPowerScreen *object, gint value);


/* ---- */

#define DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY (dbus_settings_daemon_power_screen_proxy_get_type ())
#define DBUS_SETTINGS_DAEMON_POWER_SCREEN_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY, DBusSettingsDaemonPowerScreenProxy))
#define DBUS_SETTINGS_DAEMON_POWER_SCREEN_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY, DBusSettingsDaemonPowerScreenProxyClass))
#define DBUS_SETTINGS_DAEMON_POWER_SCREEN_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY, DBusSettingsDaemonPowerScreenProxyClass))
#define DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY))
#define DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY))

typedef struct _DBusSettingsDaemonPowerScreenProxy DBusSettingsDaemonPowerScreenProxy;
typedef struct _DBusSettingsDaemonPowerScreenProxyClass DBusSettingsDaemonPowerScreenProxyClass;
typedef struct _DBusSettingsDaemonPowerScreenProxyPrivate DBusSettingsDaemonPowerScreenProxyPrivate;

struct _DBusSettingsDaemonPowerScreenProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  DBusSettingsDaemonPowerScreenProxyPrivate *priv;
};

struct _DBusSettingsDaemonPowerScreenProxyClass
{
  GDBusProxyClass parent_class;
};

GType dbus_settings_daemon_power_screen_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DBusSettingsDaemonPowerScreenProxy, g_object_unref)
#endif

void dbus_settings_daemon_power_screen_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
DBusSettingsDaemonPowerScreen *dbus_settings_daemon_power_screen_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
DBusSettingsDaemonPowerScreen *dbus_settings_daemon_power_screen_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void dbus_settings_daemon_power_screen_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
DBusSettingsDaemonPowerScreen *dbus_settings_daemon_power_screen_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
DBusSettingsDaemonPowerScreen *dbus_settings_daemon_power_screen_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON (dbus_settings_daemon_power_screen_skeleton_get_type ())
#define DBUS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON, DBusSettingsDaemonPowerScreenSkeleton))
#define DBUS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON, DBusSettingsDaemonPowerScreenSkeletonClass))
#define DBUS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON, DBusSettingsDaemonPowerScreenSkeletonClass))
#define DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON))
#define DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON))

typedef struct _DBusSettingsDaemonPowerScreenSkeleton DBusSettingsDaemonPowerScreenSkeleton;
typedef struct _DBusSettingsDaemonPowerScreenSkeletonClass DBusSettingsDaemonPowerScreenSkeletonClass;
typedef struct _DBusSettingsDaemonPowerScreenSkeletonPrivate DBusSettingsDaemonPowerScreenSkeletonPrivate;

struct _DBusSettingsDaemonPowerScreenSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  DBusSettingsDaemonPowerScreenSkeletonPrivate *priv;
};

struct _DBusSettingsDaemonPowerScreenSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType dbus_settings_daemon_power_screen_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DBusSettingsDaemonPowerScreenSkeleton, g_object_unref)
#endif

DBusSettingsDaemonPowerScreen *dbus_settings_daemon_power_screen_skeleton_new (void);


G_END_DECLS

#endif /* __DBUS_BRIGHTNESS_H__ */
